/* 
 * This software is licensed under terms of the GNU GENERAL PUBLIC LICENSE
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software  is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Source code is available at http://ppcenter.free.fr
 */


#ifndef ROM_CODE_HEADER_H
#define ROM_CODE_HEADER_H


//----------------------------------------------------------------------
// Header for use in optional ROM code : prevent the user from flashing
// ROM code incompatible with its hardware.
// Typical use is SD card access ROM code.
// 
//----------------------------------------------------------------------


// GCC have alternative #pragma pack(N) syntax and old gcc version not
// support pack(push,N), also any gcc version not support it at some platform
#if defined( __GNUC__ )
#pragma pack(1)
#else
#pragma pack(push,1)
#endif

typedef struct _sc_rom_code_header_t
{
    /* Must be set to 0x42. */
    unsigned char magic_42;
    /* Must be set to 0x53. */
    unsigned char magic_53;

    /* Version numbers. */
    unsigned char version_major;
    unsigned char version_minor;

    /* Flash write address. */
    unsigned long write_address;
    /* Flash write maximum length. */
    unsigned long write_maxlen;

    /* Compatible hardware ID. */
    unsigned char compatible_hw_id;

    /* Reserved for future use. */
    unsigned char reserved01[3];


    /* Version strings. */
    char version_string1[16];
    char version_string2[16];


    /* Reserved for future use. */
    unsigned char reserved02[15];

    /* See ROMCODE_* definitions for each bit usage. */
    unsigned char romcode;
} sc_rom_code_header_t;

// GCC have alternative #pragma pack() syntax and old gcc version not
// support pack(pop), also any gcc version not support it at some platform
#if defined( __GNUC__ )
#pragma pack()
#else
#pragma pack(pop)
#endif













#endif // ROM_CODE_HEADER_H

