/* 
 * This software is licensed under terms of the GNU GENERAL PUBLIC LICENSE
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software  is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Source code is available at http://ppcenter.free.fr
 */


#ifndef SATCOM_GLOBAL_DATA_H
#define SATCOM_GLOBAL_DATA_H


//----------------------------------------------------------------------
// SatCom Global Data.
// It consists in 64 bytes (unsigned long * 16) of data put at the
// at the end of LRAM memory.
// 
// This is used in the case we can't use local data.
//  - Communication with yabause emulator.
//
// Satcom global data's structure start address is 0x002FFFC0
// 
//----------------------------------------------------------------------


// GCC have alternative #pragma pack(N) syntax and old gcc version not
// support pack(push,N), also any gcc version not support it at some platform
#if defined( __GNUC__ )
#pragma pack(1)
#else
#pragma pack(push,1)
#endif



typedef struct _sc_crappydbg_t
{
    /******************************************/
    /* ---      Yabause usage flag.        ---*/
    /* 0: yabause unused, 1: yabause used, SatLink file I/O use, 
                          2: yabause used, yabause internal file I/O use. */
    unsigned char yabause_use;                  // 0x0600 03EC
    unsigned char unused18;                     // 0x0600 03ED

    /******************************************/
    /* ---        Crappy Debugger.         ---*/
    /* Indicates the contents of the crappy debugger message. */
    unsigned char crd_contents;                 // 0x0600 03EE
    /* Incremented when a new debug message is added. */
    unsigned char crd_counter;                  // 0x0600 03EF

    /* Pointer to the current debug message. */
    unsigned long crd_msg_ptr;                  // 0x0600 03F0
    /* Buffer used when logging numbers (1 space + 10 digits + 1 null character needed). */
    char log_buffer[12];                         // 0x0600 03F4

} sc_crappydbg_t;

typedef struct _sc_global_t
{
    /******************************************/
    /* ---        SatCom Debugger.         ---*/
    /* Address to the SCD data. */
    unsigned long scd_ptr;                      // 0x002F FFF8
    /* Magic number in order to check if we can use SCD or not. */
    unsigned long scd_magic;                    // 0x002F FFFC
} sc_global_t;


// GCC have alternative #pragma pack() syntax and old gcc version not
// support pack(pop), also any gcc version not support it at some platform
#if defined( __GNUC__ )
#pragma pack()
#else
#pragma pack(pop)
#endif


///**
// * Set global data start address at the end of the SGL "user program and data" area.
//**/
//#define SC_GLOBAL_DATA_ADR (0x060C0000 - sizeof(sc_global_t))
/**
 * Set global data start address at the end of LRAM.
**/
#define SC_GLOBAL_DATA_ADR (0x00300000 - sizeof(sc_global_t))

#include "satcart.h"



/**
 * Set crappy debugger data in a safe place unlikely to be overwritten.
**/
#define SC_CRAPPYDBG_DATA_ADR 0x060003EC




#endif //SATCOM_GLOBAL_DATA_H

