/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
/* Latest source code is available at http://ppcenter.free.fr. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define RGB_SET(R,G,B) (((B)<<10)|((G)<<5)|(R) | 0x8000)


int main (int argc, char *argv[])
{
    printf("--- BMP2RAW v0.1 by cafe-alpha. ---\n");
    if(argc <= 1)
    {
        printf(" Type %s -h for help.\n", argv[0]);
        return 0;
    }
    if(argc >= 2)
    {
        if((!strcmp(argv[1], "-h"))
        || (!strcmp(argv[1], "-help"))
        || (!strcmp(argv[1], "-?")))
        {
            printf(" Usage : -rgb555 <in_file.bmp> <out_file.bin>  ->  BMP -> RGB555\n");
            return 0;
        }
    }


    if(!strcmp(argv[1], "-rgb555"))
    {
        FILE *fp;
        unsigned char*  hp;
        unsigned char   bmp_header[54];
        unsigned char   bmp_pallette[1024];
        unsigned long   bmp_w, bmp_h;
        int i, j, ii, jj, index;
        unsigned short  bmp_bitcount;
        unsigned long   bmp_size;
        unsigned char*  data;
        unsigned short* image555;
        unsigned long   image555_len;
        char* infile  = argv[2];
        char* outfile = argv[3];


        fp = fopen(infile, "rb");
        if(fp != NULL)
        {
            fread(bmp_header, sizeof(unsigned char), 54, fp);
            hp = bmp_header;
            memcpy(&bmp_w, hp+18, sizeof(bmp_w));
            memcpy(&bmp_h, hp+22, sizeof(bmp_h));
            memcpy(&bmp_bitcount, hp+28, sizeof(bmp_bitcount));

            bmp_size = bmp_w*bmp_h * bmp_bitcount/8;
            image555_len = bmp_w*bmp_h * sizeof(unsigned short);

            data = (unsigned char*)malloc(bmp_size);

            image555 = (unsigned short*)malloc(image555_len);

            // palette 256 colors
            if(bmp_bitcount == 8)
            {
                fread(bmp_pallette, 1, 1024, fp);
                fread(data, 1, bmp_size, fp);

                /* Read 8 bits DIB image */
                jj=0;
                for(j=bmp_h-1; j>=0; j--)
                {
                    ii=0;
                    for(i=0; i<(int)bmp_w; i++)
                    {
                        unsigned char b, g, r;
                        unsigned short rgb555;

                        index = 4*data[j*bmp_w+i];

                        b = bmp_pallette[index + 0] >> 3;
                        g = bmp_pallette[index + 1] >> 3;
                        r = bmp_pallette[index + 2] >> 3;
                        rgb555 = RGB_SET(r, g, b);
                        rgb555 = ((rgb555 & 0x00FF) << 8) | ((rgb555 & 0xFF00) >> 8);

                        image555[jj*bmp_w+ii] = rgb555;
                        ii++;
                    }
                    jj++;
                }
                
            // 24 bits
            }
            else if(bmp_bitcount == 24)
            {
                fread(data, 1, bmp_size, fp);
                /* Read 24 bits DIB image */
                jj=0;
                for(j=bmp_h-1; j>=0; j--)
                {
                    ii=0;
                    for(i=0; i<(int)bmp_w; i++)
                    {
                        unsigned char b, g, r;
                        unsigned short rgb555;

                        index = (j*bmp_w+i)*3;

                        b = data[index + 0] >> 3;
                        g = data[index + 1] >> 3;
                        r = data[index + 2] >> 3;
                        rgb555 = RGB_SET(r, g, b);
                        rgb555 = ((rgb555 & 0x00FF) << 8) | ((rgb555 & 0xFF00) >> 8);

                        image555[jj*bmp_w+ii] = rgb555;
                        ii++;
                    }
                    jj++;
                }
            
            }
            free(data); data = NULL;

            fclose(fp); fp = NULL;



            fp = fopen(outfile, "wb");
            if(fp)
            {
                fwrite(image555, 1, image555_len, fp);
                fclose(fp); fp = NULL;
                printf(" Wrote %d bytes (W x H = %d x %d) to file (%s).\n", image555_len, bmp_w, bmp_h, outfile);
            }
            else
            {
                fprintf(stderr, "File write error (%s) !\n", outfile);
            }

            free(image555); image555 = NULL;
        }
        else
        {
            fprintf(stderr, "File open error (%s) !\n", infile);
        }

    }


    return 0;
}



