/**
 *  mkfirm maintenance program
 *  by cafe-alpha.
 *  WWW: http://ppcenter.free.fr/
 *
 *  Based on Sega Saturn VDP1 example program
 *  by Charles MacDonald
 *  WWW: http://cgfm2.emuviews.com
 *
 *  
**/

#include "shared.h"







/* Define line filled with spaces in order to erase a line. */
char* empty_line = "                                     ";



#define DBG2_ROW_START 15
#define DBG2_ROW_COUNT 11
#define DBG2_COL_WIDTH 36

void log_clear(void)
{
    int i;
    for(i=0; i<DBG2_ROW_COUNT; i++)
    {
        _printstr(2/*col*/, DBG2_ROW_START+i, COLOR_WHITE, empty_line);
    }
}





// unsigned char signal_char(unsigned char s)
// {
    // return (s ? 'x' : '.');
// }


/** Simple menu vvv **/
#define MENU_START_ROW 7
int menu_select(pad_t* pad, const char* menu_list[], int menu_index)
{
    int i, n, l = menu_index;

    n = 0; while(menu_list[n+1] != NULL) {n++;} n++;

    for(i=0; i<n; i++)
    {
        _printstr(2, MENU_START_ROW+i, COLOR_WHITE, empty_line);
    }

    do
    {
        /* Wait for V-Blank + perform transfers for SatCom. */
        wait_vblank();

        /* Poll joypad */
        pad_read(pad);

        if((pad->delta & PAD_UP) || (pad->new & PAD_LEFT))
        {
            l = (l != 0 ? l-1 : n-1);
        }
        else if((pad->delta & PAD_DOWN) || (pad->new & PAD_RIGHT))
        {
            l = (l != (n-1) ? l+1 : 0);
        }

        _printstr(2, 6, COLOR_YELLOW, "       --- Maintenance Menu ---       ");
        for(i=0; i<n; i++)
        {
            _printstr(2, MENU_START_ROW+i, (i==l ? COLOR_BG_FG(COLOR_RED, COLOR_WHITE) : COLOR_WHITE), (char*)menu_list[i]);
        }
    }
    while(!((pad->delta & PAD_A) || (pad->delta & PAD_B) || (pad->delta & PAD_C)) );
    for(i=0; i<n; i++)
    {
        _printstr(2, MENU_START_ROW+i, COLOR_WHITE, empty_line);
    }
    if((pad->delta & PAD_A) || (pad->delta & PAD_C))
    {
        menu_index = l;
    }
    return menu_index;
}
/** Simple menu ^^^ **/


/* Menu items definitions. */
const char* main_menu_list[] = { 
    "| Get flash chip IDs  |", 
    "| USB transfer        |", 
    "| CPLD informations   |", 
    "| Test SRAM           |", 
    "| Execute firmware    |", 
    "| Exit to multiplayer |", 
    NULL};



static void ExitToMultiplayer(void)
{
    /* Exit to multiplayer. */
    (**(volatile void(**)(void))0x600026C)();
}






#define ID_ENTRY    0x9090
#define ID_EXIT     0xF0F0
#define PAGEWRITE   0xA0A0

//static volatile unsigned short* const ADDR_FLASH = (unsigned short*)0x22000000;
static volatile unsigned short* const ADDR_55 = (unsigned short*)0x2200AAAA; /* The flash sees this as 0x5555 */
static volatile unsigned short* const ADDR_AA = (unsigned short*)0x22005554; /* The flash sees this as 0x2AAA */

static void FlashUnlock(void)
{
    *ADDR_55 = 0xAAAA;
    *ADDR_AA = 0x5555;
}

static void FlashCommand(const unsigned short Command)
{
    FlashUnlock();
    *ADDR_55 = Command;
}

unsigned short FlashGetVendorID(void)
{
    unsigned short VendorID;
    static volatile unsigned short* const ADDR_VENDOR = (unsigned short*)0x22000000;

    FlashCommand(ID_ENTRY);
    WaitForVBLANKIn();
    WaitForVBLANKOut();
    WaitForVBLANKIn(); /* Bodge to wait for flash to complete the command */
    WaitForVBLANKOut();
    VendorID = *ADDR_VENDOR;
    FlashCommand(ID_EXIT);
    WaitForVBLANKIn();
    WaitForVBLANKOut();
    WaitForVBLANKIn();
    WaitForVBLANKOut();

    return VendorID;
}

unsigned short FlashGetDeviceID(void)
{
    unsigned short DeviceID;
    static volatile unsigned short* const ADDR_DEVICE = (unsigned short*)0x22000002; /* The flash sees this as 0x0001 */

    FlashCommand(ID_ENTRY);
    WaitForVBLANKIn();
    WaitForVBLANKOut();
    WaitForVBLANKIn();
    WaitForVBLANKOut();
    DeviceID = *ADDR_DEVICE;
    FlashCommand(ID_EXIT);
    WaitForVBLANKIn();
    WaitForVBLANKOut();
    WaitForVBLANKIn();
    WaitForVBLANKOut();

    return DeviceID;
}


void FlashGetIDs(int row)
{
    unsigned short VendorID, DeviceID;

    _printstr     (2, row++, COLOR_GRAY , "Get vendor ID ... [BFh:SST]");
    VendorID = FlashGetVendorID();
    _print_strhexw(2, row++, COLOR_WHITE, " Vendor ID=", VendorID);

    _printstr     (2, row++, COLOR_GRAY , "Get device ID ...");
    _printstr     (2, row++, COLOR_GRAY , "[B5h:SST39SF010A, B6h:020A, B7h:040]");
    DeviceID = FlashGetDeviceID();
    _print_strhexw(2, row++, COLOR_WHITE, " Device ID=", DeviceID);

}





void MaintenanceMenu(int pad_connected, int decomp_error)
{
    int menu_index;
    pad_t pad;
    pad.pad_connected = pad_connected;

    pad_read(&pad);

    vdp_init();
    conio_init();

    vdp_start();


    /* Default menu item : "Execute firmware" */
    menu_index = 4;
    log_clear();
    while(1)
    {
        /* Print header messages. */
        int row = 1;
        _printstr(2, row++, COLOR_TEAL, "=/     FW maintenance utility     \\=");
        _printstr(2, row++, COLOR_YELLOW, "= By cafe-alpha (ppcenter.free.fr) =");
        _printstr(2, row++, COLOR_WHITE, "Build : " __DATE__ ", " __TIME__);
        _print_strint( 2, row  , COLOR_GRAY, "pad=", pad_connected);
        _print_strint(10, row++, COLOR_GRAY, "err=", decomp_error);


        menu_index = menu_select(&pad, main_menu_list, menu_index);
        log_clear();
        row = DBG2_ROW_START;

        switch(menu_index)
        {
        default:
        case(0): // Get flash chip IDs
            FlashGetIDs(row);
            break;

        case(1): // USB transfer
            usb_dev_cart_check_io();

            /* In the case we come back from USB dev cart function, display settings need to be reset. */
            vdp_init();
            conio_init();
            vdp_start();
            break;

        case(2): // CPLD info
            _print_strhexl(2, row++, COLOR_WHITE, " Date   =", REG_CPLD_DATE);
            _print_strhexw(2, row++, COLOR_WHITE, " Version=", REG_CART_CPLD_VER);
            _print_strhexw(2, row++, COLOR_WHITE, " Reg5555=", REG_CPLD_5555);
            _print_strhexw(2, row++, COLOR_WHITE, " RegAAAA=", REG_CPLD_AAAA);
            break;

        case(3): // Test SRAM
            _print_strhexb(2, row++, COLOR_WHITE, " sram_read=", CART_SRAM_PTR[0]);
            _printstr     (2, row++, COLOR_WHITE, " sram_wrt(55h)");
            CART_SRAM_PTR[0] = 0x55;
            break;

        case(4): // Execute firmware
            //vdp_shutdown();
            //pad_shutdown();
            return;
            break;

        case(5): // Exit to multiplayer
            ExitToMultiplayer();
            break;
        }
    }
}

