
#ifndef _VDP_H_
#define _VDP_H_


/* VDP1 memory and registers */
#define VDP1_VRAM   0x25C00000
#define VDP1_FB     0x25C80000
#define TVMR        (*(vuint16 *)0x25D00000)
#define FBCR        (*(vuint16 *)0x25D00002)
#define PTMR        (*(vuint16 *)0x25D00004)
#define EWDR        (*(vuint16 *)0x25D00006)
#define EWLR        (*(vuint16 *)0x25D00008)
#define EWRR        (*(vuint16 *)0x25D0000A)
#define ENDR        (*(vuint16 *)0x25D0000C)


#define EDSR        (*(vuint16 *)0x25D00010)
#define LOPR        (*(vuint16 *)0x25D00012)
#define COPR        (*(vuint16 *)0x25D00014)
#define MODR        (*(vuint16 *)0x25D00016)





/* VDP2 memory and registers */
#define VDP2_VRAM   0x25E00000
#define VDP2_CRAM   0x25F00000
#define VDP2_REG    0x25F80000

#define TVMD        (*(vuint16 *)0x25F80000)
#define DISP    (1<<15)
#define BDCLMD  (1<<8)
#define LSMD1   (1<<7)
#define LSMD0   (1<<6)
#define VRESO1  (1<<5)
#define VRESO0  (1<<4)
#define HRESO2  (1<<2)
#define HRESO1  (1<<1)
#define HRESO0  (1<<0)
/* Display modes. Only TV modes. */
#define NOINTERLACE     (0)
#define SINGLEINTERLACE (LSMD1)
#define DOUBLEINTERLACE (LSMD1|LSMD0)
#define VRES224         (0)
#define VRES240         (VRESO0)
/* 256 lines in PAL, 240 lines in NTSC */
#define VRES256         (VRESO1|VRESO0)
#define HRES320         (0)
#define HRES352         (HRESO0)
#define HRES640         (HRESO1)
#define HRES704         (HRESO1|HRESO0)

#define EXTEN       (*(vuint16 *)0x25F80002)
#define EXLTEN  (1<<9)
#define EXSYEN  (1<<8)
#define DASEL   (1<<1)
#define EXBGEN  (1<<0)

#define TVSTAT      (*(vuint16 *)0x25F80004)
#define EXLTFG  (1<<9)
#define EXSYFG  (1<<8)
#define VBLANK  (1<<3)
#define HBLANK  (1<<2)
#define ODD     (1<<1)
#define PAL     (1<<0)

/* Sync macros */
#define WaitForVBLANKIn()  while(!(TVSTAT&VBLANK));
#define WaitForVBLANKOut() while(TVSTAT&VBLANK);
#define WaitForHBLANKIn()  while(!(TVSTAT&HBLANK));
#define WaitForHBLANKOut() while(TVSTAT&HBLANK);


#define VRSIZE      (*(vuint16 *)0x25F80006)
#define VRAMSZ  (1<<15)

#define HCNT        (*(vuint16 *)0x25F80008)
#define VCNT        (*(vuint16 *)0x25F8000A)

#define RAMCTL      (*(vuint16 *)0x25F8000E)

#define CYCA0L      (*(vuint16 *)0x25F80010)
#define CYCA0U      (*(vuint16 *)0x25F80012)
#define CYCA1L      (*(vuint16 *)0x25F80014)
#define CYCA1U      (*(vuint16 *)0x25F80016)
#define CYCB0L      (*(vuint16 *)0x25F80018)
#define CYCB0U      (*(vuint16 *)0x25F8001A)
#define CYCB1L      (*(vuint16 *)0x25F8001C)
#define CYCB1U      (*(vuint16 *)0x25F8001E)

#define BGON        (*(vuint16 *)0x25F80020)
#define R0TPON  (1<<12)
#define N3TPON  (1<<11)
#define N2TPON  (1<<10)
#define N1TPON  (1<<9)
#define N0TPON  (1<<8)
#define R1ON    (1<<5)
#define R0ON    (1<<4)
#define N3ON    (1<<3)
#define N2ON    (1<<2)
#define N1ON    (1<<1)
#define N0ON    (1<<0)

#define MZCTL       (*(vuint16 *)0x25F80022)
#define SFSEL       (*(vuint16 *)0x25F80024)
#define SFCODE      (*(vuint16 *)0x25F80026)
#define CHCTLA      (*(vuint16 *)0x25F80028)
#define CHCTLB      (*(vuint16 *)0x25F8002A)
#define BMPNA       (*(vuint16 *)0x25F8002C)
#define BMPNB       (*(vuint16 *)0x25F8002E)

#define PNCN0       (*(vuint16 *)0x25F80030)
#define PNCN1       (*(vuint16 *)0x25F80032)
#define PNCN2       (*(vuint16 *)0x25F80034)
#define PNCN3       (*(vuint16 *)0x25F80036)
#define PNCR        (*(vuint16 *)0x25F80038)
#define PLSZ        (*(vuint16 *)0x25F8003A)
#define MPOFN       (*(vuint16 *)0x25F8003C)
#define MPOFR       (*(vuint16 *)0x25F8003E)

#define MPABN0      (*(vuint16 *)0x25F80040)
#define MPCDN0      (*(vuint16 *)0x25F80042)
#define MPABN1      (*(vuint16 *)0x25F80044)
#define MPCDN1      (*(vuint16 *)0x25F80046)
#define MPABN2      (*(vuint16 *)0x25F80048)
#define MPCDN2      (*(vuint16 *)0x25F8004A)
#define MPABN3      (*(vuint16 *)0x25F8004C)
#define MPCDN3      (*(vuint16 *)0x25F8004E)

#define MPABRA      (*(vuint16 *)0x25F80050)
#define MPCDRA      (*(vuint16 *)0x25F80052)
#define MPEFRA      (*(vuint16 *)0x25F80054)
#define MPGHRA      (*(vuint16 *)0x25F80056)
#define MPIJRA      (*(vuint16 *)0x25F80058)
#define MPKLRA      (*(vuint16 *)0x25F8005A)
#define MPMNRA      (*(vuint16 *)0x25F8005C)
#define MPOPRA      (*(vuint16 *)0x25F8005E)

#define MPABRB      (*(vuint16 *)0x25F80060)
#define MPCDRB      (*(vuint16 *)0x25F80062)
#define MPEFRB      (*(vuint16 *)0x25F80064)
#define MPGHRB      (*(vuint16 *)0x25F80066)
#define MPIJRB      (*(vuint16 *)0x25F80068)
#define MPKLRB      (*(vuint16 *)0x25F8006A)
#define MPMNRB      (*(vuint16 *)0x25F8006C)
#define MPOPRB      (*(vuint16 *)0x25F8006E)

#define SCXIN0      (*(vuint16 *)0x25F80070)
#define SCXDN0      (*(vuint16 *)0x25F80072)
#define SCYIN0      (*(vuint16 *)0x25F80074)
#define SCYDN0      (*(vuint16 *)0x25F80076)
#define ZMXIN0      (*(vuint16 *)0x25F80078)
#define ZMXDN0      (*(vuint16 *)0x25F8007A)
#define ZMYIN0      (*(vuint16 *)0x25F8007C)
#define ZMYDN0      (*(vuint16 *)0x25F8007E)

#define SCXIN1      (*(vuint16 *)0x25F80080)
#define SCXDN1      (*(vuint16 *)0x25F80082)
#define SCYIN1      (*(vuint16 *)0x25F80084)
#define SCYDN1      (*(vuint16 *)0x25F80086)
#define ZMXIN1      (*(vuint16 *)0x25F80088)
#define ZMXDN1      (*(vuint16 *)0x25F8008A)
#define ZMYIN1      (*(vuint16 *)0x25F8008C)
#define ZMYDN1      (*(vuint16 *)0x25F8008E)

#define SCXN2       (*(vuint16 *)0x25F80090)
#define SCYN2       (*(vuint16 *)0x25F80092)
#define SCXN3       (*(vuint16 *)0x25F80094)
#define SCYN3       (*(vuint16 *)0x25F80096)
#define ZMCTL       (*(vuint16 *)0x25F80098)
#define SCRCTL      (*(vuint16 *)0x25F8009A)
#define VCSTAU      (*(vuint16 *)0x25F8009C)
#define VCSTAL      (*(vuint16 *)0x25F8009E)

#define LSTA0U      (*(vuint16 *)0x25F800A0)
#define LSTA0L      (*(vuint16 *)0x25F800A2)
#define LSTA1U      (*(vuint16 *)0x25F800A4)
#define LSTA1L      (*(vuint16 *)0x25F800A6)
#define LCTAU       (*(vuint16 *)0x25F800A8)
#define LCTAL       (*(vuint16 *)0x25F800AA)
#define BKTAU       (*(vuint16 *)0x25F800AC)
#define BKTAL       (*(vuint16 *)0x25F800AE)

#define RPMD        (*(vuint16 *)0x25F800B0)
#define RPRCTL      (*(vuint16 *)0x25F800B2)
#define KTCTL       (*(vuint16 *)0x25F800B4)
#define KTAOF       (*(vuint16 *)0x25F800B6)
#define OVPNRA      (*(vuint16 *)0x25F800B8)
#define OVPNRB      (*(vuint16 *)0x25F800BA)
#define RPTAU       (*(vuint16 *)0x25F800BC)
#define RPTAL       (*(vuint16 *)0x25F800BE)

#define WPSX0       (*(vuint16 *)0x25F800C0)
#define WPSY0       (*(vuint16 *)0x25F800C2)
#define WPEX0       (*(vuint16 *)0x25F800C4)
#define WPEY0       (*(vuint16 *)0x25F800C6)
#define WPSX1       (*(vuint16 *)0x25F800C8)
#define WPSY1       (*(vuint16 *)0x25F800CA)
#define WPEX1       (*(vuint16 *)0x25F800CC)
#define WPEY1       (*(vuint16 *)0x25F800CE)

#define WCTLA       (*(vuint16 *)0x25F800D0)
#define WCTLB       (*(vuint16 *)0x25F800D2)
#define WCTLC       (*(vuint16 *)0x25F800D4)
#define WCTLD       (*(vuint16 *)0x25F800D6)
#define LWTA0U      (*(vuint16 *)0x25F800D8)
#define LWTA0L      (*(vuint16 *)0x25F800DA)
#define LWTA1U      (*(vuint16 *)0x25F800DC)
#define LWTA1L      (*(vuint16 *)0x25F800DE)

#define SPCTL       (*(vuint16 *)0x25F800E0)
#define SPCCCS1 (1<<13)
#define SPCCCS0 (1<<12)
#define SPCCN2  (1<<10)
#define SPCCN1  (1<<9)
#define SPCCN0  (1<<8)
#define SPCLMD  (1<<5)
#define SPWINEN (1<<4)
#define SPTYPE3 (1<<3)
#define SPTYPE2 (1<<2)
#define SPTYPE1 (1<<1)
#define SPTYPE0 (1<<0)
#define SPRITETYPE0 (0)
#define SPRITETYPE1 (SPTYPE0)
#define SPRITETYPE2 (SPTYPE1)
#define SPRITETYPE3 (SPTYPE1|SPTYPE0)
#define SPRITETYPE4 (SPTYPE2)
#define SPRITETYPE5 (SPTYPE2|SPTYPE0)
#define SPRITETYPE6 (SPTYPE2|SPTYPE1)
#define SPRITETYPE7 (SPTYPE2|SPTYPE1|SPTYPE0)
#define SPRITETYPE8 (SPTYPE3)
#define SPRITETYPE9 (SPTYPE3|SPTYPE0)
#define SPRITETYPEA (SPTYPE3|SPTYPE1)
#define SPRITETYPEB (SPTYPE3|SPTYPE1|SPTYPE0)
#define SPRITETYPEC (SPTYPE3|SPTYPE2)
#define SPRITETYPED (SPTYPE3|SPTYPE2|SPTYPE0)
#define SPRITETYPEE (SPTYPE3|SPTYPE2|SPTYPE1)
#define SPRITETYPEF (SPTYPE3|SPTYPE2|SPTYPE1|SPTYPE0)

#define SDCTL       (*(vuint16 *)0x25F800E2)
#define CRAOFA      (*(vuint16 *)0x25F800E4)
#define CRAOFB      (*(vuint16 *)0x25F800E6)
#define LNCLEN      (*(vuint16 *)0x25F800E8)
#define SFPRMD      (*(vuint16 *)0x25F800EA)
#define CCCTL       (*(vuint16 *)0x25F800EC)
#define SFCCMD      (*(vuint16 *)0x25F800EE)

#define PRISA       (*(vuint16 *)0x25F800F0)
#define PRISB       (*(vuint16 *)0x25F800F2)
#define PRISC       (*(vuint16 *)0x25F800F4)
#define PRISD       (*(vuint16 *)0x25F800F6)
#define PRINA       (*(vuint16 *)0x25F800F8)
#define PRINB       (*(vuint16 *)0x25F800FA)
#define PRIR        (*(vuint16 *)0x25F800FC)
#define RESERVE     (*(vuint16 *)0x25F800FE)

#define CCRSA       (*(vuint16 *)0x25F80100)
#define CCRSB       (*(vuint16 *)0x25F80102)
#define CCRSC       (*(vuint16 *)0x25F80104)
#define CCRSD       (*(vuint16 *)0x25F80106)
#define CCRNA       (*(vuint16 *)0x25F80108)
#define CCRNB       (*(vuint16 *)0x25F8010A)
#define CCRR        (*(vuint16 *)0x25F8010C)
#define CCRLB       (*(vuint16 *)0x25F8010E)

#define CLOFEN      (*(vuint16 *)0x25F80110)
#define CLOFSL      (*(vuint16 *)0x25F80112)
#define COAR        (*(vuint16 *)0x25F80114)
#define COAG        (*(vuint16 *)0x25F80116)
#define COAB        (*(vuint16 *)0x25F80118)
#define COBR        (*(vuint16 *)0x25F8011A)
#define COBG        (*(vuint16 *)0x25F8011C)
#define COBB        (*(vuint16 *)0x25F8011E)




/* Global data */
extern vuint16 *vdp1_vram;
extern vuint16 *vdp1_fb;
extern vuint16 *vdp2_vram;
extern vuint16 *vdp2_cram;

/* Function prototypes */
void vdp_init(void);
void vdp_start(void);
void vdp_shutdown();
void wait_hblank(void);
void wait_hblank_in(void);
void wait_hblank_out(void);
void wait_vblank_in(void);
void wait_vblank_out(void);
void wait_vblank(void);

#endif /* _VDP_H_ */

